# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: PublicFuture.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC, 6, 31, 1, "", "PublicFuture.proto"
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12PublicFuture.proto"\xd3\x0b\n\x0f\x43ontractWrapper\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0f\n\x07\x63hannel\x18\x02 \x01(\t\x12\n\n\x02ts\x18\x03 \x01(\x03\x12$\n\x0borderChange\x18\xe9\x07 \x01(\x0b\x32\x0c.OrderChangeH\x00\x12$\n\x0b\x61ssetChange\x18\xea\x07 \x01(\x0b\x32\x0c.AssetChangeH\x00\x12$\n\x0b\x62onusNotify\x18\xeb\x07 \x01(\x0b\x32\x0c.BonusNotifyH\x00\x12&\n\x0cleverageMode\x18\xec\x07 \x01(\x0b\x32\r.LeverageModeH\x00\x12(\n\rnotifyGeneric\x18\xed\x07 \x01(\x0b\x32\x0e.NotifyGenericH\x00\x12\x34\n\x13liquidateRiskChange\x18\xee\x07 \x01(\x0b\x32\x14.LiquidateRiskChangeH\x00\x12,\n\x0fmarketOrderDeal\x18\xef\x07 \x01(\x0b\x32\x10.MarketOrderDealH\x00\x12,\n\x0forderDealDetail\x18\xf0\x07 \x01(\x0b\x32\x10.OrderDealDetailH\x00\x12\x30\n\x11\x63ontractPlanOrder\x18\xf1\x07 \x01(\x0b\x32\x12.ContractPlanOrderH\x00\x12\x32\n\x12positionModeResult\x18\xf2\x07 \x01(\x0b\x32\x13.PositionModeResultH\x00\x12*\n\x0epositionChange\x18\xf3\x07 \x01(\x0b\x32\x0f.PositionChangeH\x00\x12\x38\n\x15positionReverseChange\x18\xf4\x07 \x01(\x0b\x32\x16.PositionReverseChangeH\x00\x12*\n\x0euserPreference\x18\xf5\x07 \x01(\x0b\x32\x0f.UserPreferenceH\x00\x12\x30\n\x11\x63ontractStopOrder\x18\xf6\x07 \x01(\x0b\x32\x12.ContractStopOrderH\x00\x12\x38\n\x15\x63ontractUserRiskLimit\x18\xf7\x07 \x01(\x0b\x32\x16.ContractUserRiskLimitH\x00\x12\x38\n\x15\x63ontractStopPlanOrder\x18\xf8\x07 \x01(\x0b\x32\x16.ContractStopPlanOrderH\x00\x12\x32\n\x12\x63ontractTrackOrder\x18\xf9\x07 \x01(\x0b\x32\x13.ContractTrackOrderH\x00\x12"\n\norderChase\x18\xfa\x07 \x01(\x0b\x32\x0b.OrderChaseH\x00\x12"\n\nindexPrice\x18\xd1\x0f \x01(\x0b\x32\x0b.IndexPriceH\x00\x12&\n\x0c\x63ommonConfig\x18\xd2\x0f \x01(\x0b\x32\r.CommonConfigH\x00\x12(\n\rcontractExtra\x18\xd3\x0f \x01(\x0b\x32\x0e.ContractExtraH\x00\x12\x1e\n\x08\x63ontract\x18\xd4\x0f \x01(\x0b\x32\t.ContractH\x00\x12\x16\n\x04\x64\x65\x61l\x18\xd5\x0f \x01(\x0b\x32\x05.DealH\x00\x12\x18\n\x05\x64\x65\x61ls\x18\xd6\x0f \x01(\x0b\x32\x06.DealsH\x00\x12\x32\n\x12marketOrderSetting\x18\xb4\x10 \x01(\x0b\x32\x13.MarketOrderSettingH\x00\x12\x1c\n\x07wsKline\x18\xb5\x10 \x01(\x0b\x32\x08.WsKLineH\x00\x12$\n\x0b\x66undingRate\x18\xb6\x10 \x01(\x0b\x32\x0c.FundingRateH\x00\x12 \n\tfairPrice\x18\xdd\x0f \x01(\x0b\x32\n.FairPriceH\x00\x12(\n\rdepthStepInfo\x18\xde\x0f \x01(\x0b\x32\x0e.DepthStepInfoH\x00\x12 \n\tdepthInfo\x18\xdf\x0f \x01(\x0b\x32\n.DepthInfoH\x00\x12"\n\ndepthMerge\x18\xe0\x0f \x01(\x0b\x32\x0b.DepthMergeH\x00\x12&\n\x0ctickerDetail\x18\xe1\x0f \x01(\x0b\x32\r.TickerDetailH\x00\x12"\n\ntickerList\x18\xe2\x0f \x01(\x0b\x32\x0b.TickerListH\x00\x12\x1e\n\x08tickList\x18\xe3\x0f \x01(\x0b\x32\t.TickListH\x00\x42\t\n\x07payload"M\n\tDepthInfo\x12\x14\n\x04\x62ids\x18\x01 \x03(\x0b\x32\x06.Order\x12\x14\n\x04\x61sks\x18\x02 \x03(\x0b\x32\x06.Order\x12\x14\n\x0clastUpdateId\x18\x03 \x01(\x03"8\n\nDepthMerge\x12\x14\n\x04\x62ids\x18\x01 \x03(\x0b\x32\x06.Order\x12\x14\n\x04\x61sks\x18\x02 \x03(\x0b\x32\x06.Order"(\n\x05Order\x12\r\n\x05price\x18\x01 \x01(\t\x12\x10\n\x08quantity\x18\x02 \x01(\t"\x99\x03\n\x0cTickerDetail\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x11\n\tlastPrice\x18\x02 \x01(\t\x12\x14\n\x0criseFallRate\x18\x03 \x01(\t\x12\x11\n\tfairPrice\x18\x04 \x01(\t\x12\x12\n\nindexPrice\x18\x05 \x01(\t\x12\x10\n\x08volume24\x18\x06 \x01(\t\x12\x10\n\x08\x61mount24\x18\x07 \x01(\t\x12\x13\n\x0bmaxBidPrice\x18\x08 \x01(\t\x12\x13\n\x0bminAskPrice\x18\t \x01(\t\x12\x14\n\x0clower24Price\x18\n \x01(\t\x12\x13\n\x0bhigh24Price\x18\x0b \x01(\t\x12\x11\n\ttimestamp\x18\x0c \x01(\x03\x12\x0c\n\x04\x62id1\x18\r \x01(\t\x12\x0c\n\x04\x61sk1\x18\x0e \x01(\t\x12\x0f\n\x07holdVol\x18\x0f \x01(\t\x12\x15\n\rriseFallValue\x18\x10 \x01(\t\x12\x13\n\x0b\x66undingRate\x18\x11 \x01(\t\x12\x0c\n\x04zone\x18\x12 \x01(\t\x12\x15\n\rriseFallRates\x18\x13 \x03(\t\x12\x1f\n\x17riseFallRatesOfTimezone\x18\x14 \x03(\t",\n\nTickerList\x12\x1e\n\x07tickers\x18\x01 \x03(\x0b\x32\r.TickerDetail"D\n\x0b\x46undingRate\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04rate\x18\x02 \x01(\t\x12\x17\n\x0fnextFundingTime\x18\x03 \x01(\x03"+\n\nIndexPrice\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t"*\n\tFairPrice\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\r\n\x05price\x18\x02 \x01(\t"O\n\x0bOrderChange\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"3\n\x0b\x41ssetChange\x12\r\n\x05\x61sset\x18\x01 \x01(\t\x12\x15\n\rbalanceChange\x18\x02 \x01(\t"S\n\x0fMarketOrderDeal\x12\x0e\n\x06\x64\x65\x61lId\x18\x01 \x01(\t\x12\x0f\n\x07orderId\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"F\n\x0ePositionChange\x12\x12\n\npositionId\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t\x12\x10\n\x08quantity\x18\x03 \x01(\t"\x1d\n\x05\x44\x65\x61ls\x12\x14\n\x05\x64\x65\x61ls\x18\x01 \x03(\x0b\x32\x05.Deal"I\n\x04\x44\x65\x61l\x12\t\n\x01p\x18\x01 \x01(\t\x12\t\n\x01v\x18\x02 \x01(\t\x12\t\n\x01t\x18\x03 \x01(\x05\x12\t\n\x01o\x18\x04 \x01(\x05\x12\t\n\x01m\x18\x05 \x01(\x05\x12\n\n\x02ts\x18\x06 \x01(\x03"L\n\x0b\x42onusNotify\x12\x0f\n\x07\x62onusId\x18\x01 \x01(\t\x12\x0c\n\x04type\x18\x02 \x01(\t\x12\x0e\n\x06\x61mount\x18\x03 \x01(\t\x12\x0e\n\x06status\x18\x04 \x01(\t",\n\x0cLeverageMode\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04mode\x18\x02 \x01(\t"A\n\rNotifyGeneric\x12\x11\n\tmessageId\x18\x01 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t"L\n\x13LiquidateRiskChange\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x11\n\triskLevel\x18\x02 \x01(\t\x12\x12\n\nmarginCall\x18\x03 \x01(\t"T\n\x0fOrderDealDetail\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0f\n\x07tradeId\x18\x02 \x01(\t\x12\r\n\x05price\x18\x03 \x01(\t\x12\x10\n\x08quantity\x18\x04 \x01(\t"4\n\x11\x43ontractPlanOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\t""\n\x12PositionModeResult\x12\x0c\n\x04mode\x18\x01 \x01(\t"A\n\x15PositionReverseChange\x12\x12\n\npositionId\x18\x01 \x01(\t\x12\x14\n\x0c\x63hangeAmount\x18\x02 \x01(\t"2\n\x0eUserPreference\x12\x0e\n\x06userId\x18\x01 \x01(\t\x12\x10\n\x08settings\x18\x02 \x01(\t"7\n\x11\x43ontractStopOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x11\n\tstopPrice\x18\x02 \x01(\t":\n\x15\x43ontractUserRiskLimit\x12\x0e\n\x06userId\x18\x01 \x01(\t\x12\x11\n\triskLimit\x18\x02 \x01(\t">\n\x15\x43ontractStopPlanOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x14\n\x0ctriggerPrice\x18\x02 \x01(\t"9\n\x12\x43ontractTrackOrder\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x12\n\ntrackPrice\x18\x02 \x01(\t"1\n\nOrderChase\x12\x0f\n\x07orderId\x18\x01 \x01(\t\x12\x12\n\nchasePrice\x18\x02 \x01(\t"*\n\x0c\x43ommonConfig\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"5\n\rContractExtra\x12\x12\n\ncontractId\x18\x01 \x01(\t\x12\x10\n\x08leverage\x18\x02 \x01(\t".\n\x08\x43ontract\x12\x12\n\ncontractId\x18\x01 \x01(\t\x12\x0e\n\x06symbol\x18\x02 \x01(\t"6\n\x12MarketOrderSetting\x12\x11\n\tsettingId\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"a\n\x07WsKLine\x12\x0e\n\x06symbol\x18\x01 \x01(\t\x12\x0c\n\x04open\x18\x02 \x01(\t\x12\x0c\n\x04high\x18\x03 \x01(\t\x12\x0b\n\x03low\x18\x04 \x01(\t\x12\r\n\x05\x63lose\x18\x05 \x01(\t\x12\x0e\n\x06volume\x18\x06 \x01(\t",\n\rDepthStepInfo\x12\x0c\n\x04step\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\t"(\n\x08TickList\x12\x1c\n\x05ticks\x18\x01 \x03(\x0b\x32\r.TickerDetailb\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "PublicFuture_pb2", _globals)
if not _descriptor._USE_C_DESCRIPTORS:
    DESCRIPTOR._loaded_options = None
    _globals["_CONTRACTWRAPPER"]._serialized_start = 23
    _globals["_CONTRACTWRAPPER"]._serialized_end = 1514
    _globals["_DEPTHINFO"]._serialized_start = 1516
    _globals["_DEPTHINFO"]._serialized_end = 1593
    _globals["_DEPTHMERGE"]._serialized_start = 1595
    _globals["_DEPTHMERGE"]._serialized_end = 1651
    _globals["_ORDER"]._serialized_start = 1653
    _globals["_ORDER"]._serialized_end = 1693
    _globals["_TICKERDETAIL"]._serialized_start = 1696
    _globals["_TICKERDETAIL"]._serialized_end = 2105
    _globals["_TICKERLIST"]._serialized_start = 2107
    _globals["_TICKERLIST"]._serialized_end = 2151
    _globals["_FUNDINGRATE"]._serialized_start = 2153
    _globals["_FUNDINGRATE"]._serialized_end = 2221
    _globals["_INDEXPRICE"]._serialized_start = 2223
    _globals["_INDEXPRICE"]._serialized_end = 2266
    _globals["_FAIRPRICE"]._serialized_start = 2268
    _globals["_FAIRPRICE"]._serialized_end = 2310
    _globals["_ORDERCHANGE"]._serialized_start = 2312
    _globals["_ORDERCHANGE"]._serialized_end = 2391
    _globals["_ASSETCHANGE"]._serialized_start = 2393
    _globals["_ASSETCHANGE"]._serialized_end = 2444
    _globals["_MARKETORDERDEAL"]._serialized_start = 2446
    _globals["_MARKETORDERDEAL"]._serialized_end = 2529
    _globals["_POSITIONCHANGE"]._serialized_start = 2531
    _globals["_POSITIONCHANGE"]._serialized_end = 2601
    _globals["_DEALS"]._serialized_start = 2603
    _globals["_DEALS"]._serialized_end = 2632
    _globals["_DEAL"]._serialized_start = 2634
    _globals["_DEAL"]._serialized_end = 2707
    _globals["_BONUSNOTIFY"]._serialized_start = 2709
    _globals["_BONUSNOTIFY"]._serialized_end = 2785
    _globals["_LEVERAGEMODE"]._serialized_start = 2787
    _globals["_LEVERAGEMODE"]._serialized_end = 2831
    _globals["_NOTIFYGENERIC"]._serialized_start = 2833
    _globals["_NOTIFYGENERIC"]._serialized_end = 2898
    _globals["_LIQUIDATERISKCHANGE"]._serialized_start = 2900
    _globals["_LIQUIDATERISKCHANGE"]._serialized_end = 2976
    _globals["_ORDERDEALDETAIL"]._serialized_start = 2978
    _globals["_ORDERDEALDETAIL"]._serialized_end = 3062
    _globals["_CONTRACTPLANORDER"]._serialized_start = 3064
    _globals["_CONTRACTPLANORDER"]._serialized_end = 3116
    _globals["_POSITIONMODERESULT"]._serialized_start = 3118
    _globals["_POSITIONMODERESULT"]._serialized_end = 3152
    _globals["_POSITIONREVERSECHANGE"]._serialized_start = 3154
    _globals["_POSITIONREVERSECHANGE"]._serialized_end = 3219
    _globals["_USERPREFERENCE"]._serialized_start = 3221
    _globals["_USERPREFERENCE"]._serialized_end = 3271
    _globals["_CONTRACTSTOPORDER"]._serialized_start = 3273
    _globals["_CONTRACTSTOPORDER"]._serialized_end = 3328
    _globals["_CONTRACTUSERRISKLIMIT"]._serialized_start = 3330
    _globals["_CONTRACTUSERRISKLIMIT"]._serialized_end = 3388
    _globals["_CONTRACTSTOPPLANORDER"]._serialized_start = 3390
    _globals["_CONTRACTSTOPPLANORDER"]._serialized_end = 3452
    _globals["_CONTRACTTRACKORDER"]._serialized_start = 3454
    _globals["_CONTRACTTRACKORDER"]._serialized_end = 3511
    _globals["_ORDERCHASE"]._serialized_start = 3513
    _globals["_ORDERCHASE"]._serialized_end = 3562
    _globals["_COMMONCONFIG"]._serialized_start = 3564
    _globals["_COMMONCONFIG"]._serialized_end = 3606
    _globals["_CONTRACTEXTRA"]._serialized_start = 3608
    _globals["_CONTRACTEXTRA"]._serialized_end = 3661
    _globals["_CONTRACT"]._serialized_start = 3663
    _globals["_CONTRACT"]._serialized_end = 3709
    _globals["_MARKETORDERSETTING"]._serialized_start = 3711
    _globals["_MARKETORDERSETTING"]._serialized_end = 3765
    _globals["_WSKLINE"]._serialized_start = 3767
    _globals["_WSKLINE"]._serialized_end = 3864
    _globals["_DEPTHSTEPINFO"]._serialized_start = 3866
    _globals["_DEPTHSTEPINFO"]._serialized_end = 3910
    _globals["_TICKLIST"]._serialized_start = 3912
    _globals["_TICKLIST"]._serialized_end = 3952
# @@protoc_insertion_point(module_scope)
